

PROGRAM splitArrays
  USE types
  IMPLICIT NONE
#include "finclude/petsc.h"
!#include "mpif.h"
  Mat :: A,B
  PetscViewer :: matviewer
  PetscErrorCode :: perr
  INTEGER(i4), DIMENSION(:), ALLOCATABLE :: annz,acols
  REAL(r8), DIMENSION(:), ALLOCATABLE :: adata
  INTEGER(i4), DIMENSION(:), ALLOCATABLE :: bdnnz,bdcols
  REAL(r8), DIMENSION(:), ALLOCATABLE :: bddata
  INTEGER(i4), DIMENSION(:), ALLOCATABLE :: bonnz,bocols
  REAL(r8), DIMENSION(:), ALLOCATABLE :: bodata
  INTEGER(i4) :: nArgs,dofs,nprocs,rank,localRows,globalRow,entries
  INTEGER(i4) :: numNonzeros
  INTEGER(i4) :: i,j,nOn,nOff,numDiags
  INTEGER(i4), DIMENSION(:), ALLOCATABLE :: globalRows
  CHARACTER(12) :: str
  INTEGER(i4) :: ierror
  KSP :: ksp
  Mat :: realD
  call PetscInitialize(PETSC_NULL_CHARACTER,ierror)
  call mpi_comm_rank(mpi_comm_world,rank,ierror)
  call mpi_comm_size(mpi_comm_world,nprocs,ierror)

  ALLOCATE(globalRows(0:nprocs-1))

  nArgs = iargc()
  call getarg(1,str)
  read(str,'(i4)') dofs

  ! divvy up the dofs
  if (rank .ne. 0) then
    localRows = dofs/nprocs
  else
    localRows = dofs - dofs/nprocs * (nprocs-1)
  endif
  ALLOCATE(annz(localRows+1))

  call mpi_allgather(localRows,1,mpi_integer,globalRows(0),1,&
                     mpi_integer,mpi_comm_world,ierror)

  if (rank.eq.0) then
    numNonzeros=2+3*(localRows-1)
  else if (rank .eq. nprocs-1) then
    numNonzeros=2+3*(localRows-1)
  else
    numNonzeros=3*localRows
  endif 
  
  ALLOCATE(acols(numNonzeros),adata(numNonzeros))

  annz(1)=0_i4
  entries=0
  do i=1,localRows
    globalRow=i+sum(globalRows(0:rank-1))-1
    if (globalRow.eq.0) then
       annz(i+1)=annz(i)+2
       acols(entries+1)=globalRow
       adata(entries+1)=-2._r8
       acols(entries+2)=globalRow+1
       adata(entries+2)=1._r8
       entries=entries+2
    else if (globalRow.eq.dofs-1) then
       annz(i+1)=annz(i)+2
       acols(entries+1)=globalRow-1
       adata(entries+1)=1._r8
       acols(entries+2)=globalRow
       adata(entries+2)=-2._r8
       entries=entries+2
    else
       annz(i+1)=annz(i)+3
       acols(entries+1)=globalRow-1
       adata(entries+1)=1._r8
       acols(entries+2)=globalRow
       adata(entries+2)=-2._r8
       acols(entries+3)=globalRow+1
       adata(entries+3)=1._r8
       entries=entries+3
    endif
  enddo

  call MatCreate(PETSC_COMM_WORLD,A,perr)
  call MatSetSizes(A,localRows,localRows,dofs,dofs,perr)
  call MatSetType(A,'mpiaij',perr)
  call MatMPIAIJSetPreallocationCSR(A,annz,acols,adata,perr)
  call PetscViewerCreate(PETSC_COMM_WORLD,matviewer,perr)
  call PetscViewerSetType(matviewer,PETSC_VIEWER_ASCII,perr)
  call PetscViewerSetFormat(matviewer,PETSC_VIEWER_ASCII_MATLAB,perr)
  call PetscViewerFileSetName(matviewer,'matA.dat',perr)
  call MatView(A,matviewer,perr)
  call PetscViewerDestroy(matviewer,perr)

  ! Now transfer to WithSplitArrays
  ALLOCATE(bdnnz(localRows+1),bonnz(localRows+1))
  ! Count off diagonal entries
  entries=0
  do i=1,numNonzeros
    if (acols(i) .gt. sum(globalRows(0:rank-1)) + localRows-1 .or. &
        acols(i) .lt. sum(globalRows(0:rank-1))) entries=entries+1
  enddo
  numDiags=numNonzeros-entries
  ALLOCATE(bocols(entries),bodata(entries))
  ALLOCATE(bdcols(numDiags),bddata(numDiags))
  nOff=0
  nOn=0
  bonnz(1)=nOff
  bdnnz(1)=nOn
  do i=1,localRows
    write(*,*) 'jbegin,jend=',annz(i)+1,annz(i+1)
    do j=annz(i)+1,annz(i+1)
      if (acols(j) .gt. sum(globalRows(0:rank-1)) + localRows-1 .or. &
          acols(j) .lt. sum(globalRows(0:rank-1))) then
        nOff=nOff+1
        bocols(nOff)=acols(j)
        bodata(nOff)=adata(j)
      else
        nOn=nOn+1
        bdcols(nOn)=acols(j)
        bddata(nOn)=adata(j)
      endif
    enddo 
    bdnnz(i+1)=nOn
    bonnz(i+1)=nOff
  enddo 

  if (rank.gt.0) bdcols=bdcols-sum(globalRows(0:rank-1))

  call MatCreateMPIAIJWithSplitArrays(PETSC_COMM_WORLD,&
                  localRows,localRows,dofs,dofs,       &
                  bdnnz,bdcols,bddata,bonnz,bocols,bodata,B,perr)

  call PetscViewerCreate(PETSC_COMM_WORLD,matviewer,perr)
  call PetscViewerSetType(matviewer,PETSC_VIEWER_ASCII,perr)
  call PetscViewerSetFormat(matviewer,PETSC_VIEWER_ASCII_MATLAB,perr)
  call PetscViewerFileSetName(matviewer,'matB.dat',perr)
  call MatView(B,matviewer,perr)
  call PetscViewerDestroy(matviewer,perr)
  ! NIMROD test
  call KSPCreate(PETSC_COMM_WORLD,ksp,perr)
  call KSPSetFromOptions(ksp,perr)
  call KSPSetOperators(ksp,B,B,SAME_PRECONDITIONER,perr)
  call KSPGetOperators(ksp,realD,realD,&
            PETSC_NULL_INTEGER,perr)
!  DEALLOCATE(bdnnz)
!  DEALLOCATE(bdcols)
!  DEALLOCATE(bddata)
!  DEALLOCATE(bonnz)
!  DEALLOCATE(bocols)
!  DEALLOCATE(bodata)
  call MatDestroy(B,perr)
  ! Now try to access the (hopefully) deallocated data
  write(*,*) bdnnz

  call PetscFinalize(perr)

END PROGRAM splitArrays
